#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import os

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Labels of data sources

NVD_DATA_SRC   = 'nvd'
FSTEC_DATA_SRC = 'fstec'
DATA_SOURCES = (NVD_DATA_SRC, FSTEC_DATA_SRC)
VUL_MARKERS  = {NVD_DATA_SRC: 'CVE-', FSTEC_DATA_SRC: 'BDU:'}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Conf dir and conf files paths

CVE_MANAGER = 'cve-manager'
CVE_MONITOR = 'cve-monitor'
CVE_MAIL    = 'cve-mail'
CVE_MANAGER_UNITS = (CVE_MANAGER, CVE_MONITOR, CVE_MAIL)
CONF_DIR = '/etc/cve-manager/'
CONF_DIR_DEBUG = os.path.expanduser('~/devel/cve-manager/samples/')
CONF_FILES = \
	{m: os.path.join(CONF_DIR, f'{m}.conf') for m in CVE_MANAGER_UNITS}
CONF_FILES_DEBUG = \
	{m: os.path.join(CONF_DIR_DEBUG, f'{m}.debug.conf') for m in CVE_MANAGER_UNITS}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Possible download targets

DT_NVD_VUL = 'nvd_vul'
DT_CPE     = 'cpe'
DT_ACL     = 'acl'
DT_TIMELINES  = 'timelines'
DT_FSTEC_VUL  = 'fstec_vul'
DT_DISTRO_SRC = 'distro_lists_src'
DT_DISTRO_BIN = 'distro_lists_bin'
REQUIRED_DOWNLOADS = [DT_NVD_VUL, DT_CPE, DT_ACL, DT_TIMELINES]
OPTIONAL_DOWNLOADS = [DT_FSTEC_VUL, DT_DISTRO_SRC, DT_DISTRO_BIN]
ALL_DOWNLOADS = REQUIRED_DOWNLOADS + OPTIONAL_DOWNLOADS

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Available types of matching of CPE product names to package names

M_PRESCRIBED = 'prescribed'
M_URL = 'url'
M_COMPLETE = 'complete'
M_PARTIAL  = 'partial'
M_BIN_COMPLETE = 'bincomplete'
M_BIN_PARTIAL  = 'binpartial'
M_FIXES = 'fixes'
M_CHOICES = [
	M_PRESCRIBED,
	M_URL,
	M_COMPLETE,
	M_PARTIAL,
	M_FIXES,
	M_BIN_COMPLETE,
	M_BIN_PARTIAL,
	]
