#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import multiprocessing as mp
from collections import defaultdict

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

def Parallel(target, iterable, extra_params):

	# Starting multiple processes with assigned portions of data to handle,
	# using pipes for results and event object for error signalization
	processes  = []
	recv_pipes = []
	err_event  = mp.Event()
	for entry in iterable:
		recv_pipe, send_pipe = mp.Pipe(duplex=False)
		p = mp.Process(target=target,
			args=(entry, extra_params, send_pipe, err_event))
		p.start()
		processes.append(p)
		recv_pipes.append(recv_pipe)

	# Waiting the processes to finish
	for p in processes:
		p.join()

	# Getting feedback messages from the pipes
	feedback = None
	err = None
	for recv_pipe in recv_pipes:
		transfer = recv_pipe.recv()
		t = type(transfer)
		if feedback == None:
			feedback = t(transfer.default_factory) if t is defaultdict else t()
		elif type(feedback) is not t:
			err = f'Different types of pipe messages: {transfer_type} and {t}'
			break
		if t is list:
			for entry in transfer:
				feedback.append(entry)
		elif t is dict:
			for k, v in transfer.items():
				feedback[k] = v
		elif t is defaultdict:
			for k1, v1 in transfer.items():
				for k2, v2 in v1.items():
					feedback[k1][k2] = v2
		else:
			err = 'Unsupported type of pipe messages'
			break

	if err:
		raise Exception(err)
		return None, False

	# Inspecting the event flag
	ok = not err_event.is_set()

	return feedback, ok
