#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Split given software listing that looks like "a&&b&&c d&&e g ..." and product
# a list of lists of "Software" objects out of it

def SplitSoftware(software_listing):

	return [[Software(el) for el in conjunction.split('&&')]
		for conjunction in software_listing.split()]

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Split given software listing that looks like "a&&b&&c d&&e g ..." and produce
# a set of "Software" objects out of it

def DistinctSoftware(software_listing):

	distinct_cpe = {el for conjunction in software_listing.split()
		for el in conjunction.split('&&')}

	return [Software(el) for el in distinct_cpe]

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Full NVD or FSTEC name

class Software():

	def __init__(self, full_name):

		self.software = []

		if full_name and full_name[0] == '~':
			self.vulnerable = False
			full_name = full_name[1:]
		else:
			self.vulnerable = True

		for i, s in enumerate(full_name.split(':')):
			if i > 2:
				break
			self.software.append(s)

		l = len(self.software)
		d = 3 - l
		if d > 0:
			self.software[l:(l + d)] = [''] * d

	# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

	def IsVulnerable(self):
		return self.vulnerable

	def Vendor(self):
		return self.software[0]

	def Product(self):
		return self.software[1]

	def Ver(self):
		return self.software[2]
