#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import re

NETLOC = 'debian.org'
NETLOC_LEN = len(NETLOC)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Format a Debian URL

def Fmt(netloc, path, query):

	if path:
		path = re.sub(r'\.app$', '', path)

	steps = []
	for step in path.split('/'):
		if not step:
			continue
		if not re.search(r'[a-z]', step) or \
				step in ('tags', 'news', 'blob', 'master', 'changes'):
			break
		steps.append(step)

	n_steps = len(steps)
	prefix  = netloc[:-(NETLOC_LEN + 1)] if len(netloc) > NETLOC_LEN else ''
	product = None

	# Processing Debian URLs of the following special formats:
	# _ (packages|...|sources).debian.org/.../<product>
	# _ (packages|...|sources).debian.org/search?keywords=<product>
	if prefix in ('packages', 'packages.qa', 'people', 'ftp', 'salsa', 'sources'):
		if steps:
			if steps[-1] == 'search':
				match = re.match(r'keywords=([^,;&]+)', query)
				product = match.group(1) if match else ''
			else:
				product = steps[-1]
		else:
			product = ''
	# _ tracker.debian.org/pkg/<product>/...
	# _ tracker.debian.org/.../packages/<char>/<product>/...
	elif prefix == 'tracker':
		for regex in (r'/pkg/([^/]+)', r'.*/packages/[^/]/([^/]+)'):
			match = re.match(regex, path)
			if match:
				product = match.group(1)
				break
	# _ metadata.ftp-master.debian.org/changelogs/main/<char>/<product>
	# _ ftp-master.metadata.debian.org/changelogs/main/<char>/<product>
	elif prefix in ('metadata.ftp-master', 'ftp-master.metadata'):
		if n_steps >= 4:
			product = steps[3]
	# _ (git|(...)?alioth).debian.org/?p=(.../)?<product>
	# _ (git|(...)?alioth).debian.org/projects/<product>
	# _ <product>.alioth.debian.org/...
	elif prefix == 'git' or re.search(r'(^|\.)(alioth|anonscm)$', prefix):
		if n_steps >= 2 and steps[0] == 'projects':
			product = steps[1]
		else:
			match = re.match(r'p=([^,;&]+)', query)
			if match:
				product = re.sub(r'\.git$', '', match.group(1).split('/')[-1])
			else:
				match = re.match(r'^([^\.]+)\.', prefix)
				if match:
					product = match.group(1)
	# _ security-tracker.debian.org/tracker/source-package/<product>
	elif prefix == 'security-tracker':
		if n_steps >= 3 and steps[1] == 'source-package':
			product = steps[2]
	# _ bugs.debian.org/.../?package=<product>
	elif prefix == 'bugs':
		match = re.match(r'package=(.+)', query)
		if match:
			product = match.group(1)
	# _ wiki.debian.org/<product>
	elif prefix == 'wiki':
		product = steps[0] if steps else ''

	# Final formatting
	if product != None:
		netloc = f'{product}.{NETLOC}' if product else netloc
		path  = ''
		query = ''

	return netloc, path, query
