#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import re

NETLOC = 'fedorahosted.org'
NETLOC_LEN = len(NETLOC)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Format a FedoraHosted URL

def Fmt(netloc, path, query):

	steps   = [s for s in path.split('/') if s]
	n_steps = len(steps)
	prefix  = netloc[:-(NETLOC_LEN + 1)] if len(netloc) > NETLOC_LEN else ''
	product = None

	# Processing FedoraHosted URLs of the following special formats:
	# _ <product>.fedorahosted.org/...
	# _ git.fedorahosted.org/git/<macro_project>/<product>.git/...
	# _ git.fedorahosted.org/cgit/<product>.git/...
	if prefix:
		if prefix == 'git':
			if n_steps >= 2:
				if steps[0] == 'git':
					product = steps[2] if n_steps >= 3 else steps[1]
				elif steps[0] == 'cgit':
					product = steps[1]
				if product:
					product = re.sub(r'\.git$', '', product)
			else:
				netloc = NETLOC
		else:
			product = prefix
	# _ fedorahosted.org/<product>
	# _ fedorahosted.org/<product>/wiki/...
	# _ fedorahosted.org/released/<product>/...
	# _ fedorahosted.org/releases/<char>/<char>/<product>/...
	elif steps:
		if n_steps >= 2:
			if steps[0] == 'releases':
				if n_steps >= 4:
					product = steps[3]
			elif steps[0] == 'released':
				product = steps[1]
			elif steps[1] == 'wiki':
				product = steps[0]
		elif steps[0] not in ('released', 'releases'):
			product = steps[0]

	# Final formatting
	if product != None:
		netloc = f'{product}.{NETLOC}' if product else netloc
		path  = ''
		query = ''

	return netloc, path, query
