#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import re

NETLOC = 'freedesktop.org'
NETLOC_LEN = len(NETLOC)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Format a FreeDesktop URL

def Fmt(netloc, path, query):

	if not path:
		return netloc, '', ''

	steps = [s for s in path.split('/') if s and re.search(r'[a-z]', s)]
	if not steps:
		return netloc, '', ''

	n_steps = len(steps)
	prefix  = netloc[:-(NETLOC_LEN + 1)] if len(netloc) > NETLOC_LEN else ''
	product = None

	# Processing FreeDesktop URLs of the following special formats:
	# _ (<product>.)?freedesktop.org/(wiki|software)/.../<product>
	if steps[0] in ('wiki', 'software'):
		product = steps[-1] if n_steps >= 2 else ''
	# _ .*freedesktop.org/(modules|components|docs|src)/<product>.*
	elif re.search('^(modules?|components?|docs?|src)$', steps[0]):
		product = steps[1] if n_steps >= 2 else ''
	# _ .*freedesktop.org/releases/<product>/...
	elif re.search('^releases?$', steps[0]):
		product = steps[1] if n_steps >= 2 and steps[1] != 'individual' else ''
	# _ .*freedesktop.org/libreoffice/<sub_project>/...
	elif steps[0] == 'libreoffice':
		product = steps[0]
	# _ (cgit|gitlab).freedesktop.org/(<subdir>/)?(lib/|...|linux/)?<product>.*
	elif prefix in ('cgit', 'gitlab'):
		if n_steps >= 2 and steps[1] not in \
				('summary', 'refs', 'log', 'tree', 'commit', 'diff'):
			if steps[1] not in ('lib', 'app', 'test', 'driver', 'linux'):
				product = steps[1]
			else:
				product = steps[2] if n_steps >= 3 else steps[0]
		else:
			product = steps[0]
	# _ lists.freedesktop.org/<subdir>/<product>.*
	elif prefix in ('lists', 'people'):
		product = steps[1] if n_steps >= 2 else ''

	# Final formatting
	if product != None:
		netloc = f'{product}.{NETLOC}' if product else netloc
		path  = ''
		query = ''
	else:
		path = '/' + '/'.join(steps)

	return netloc, path, query
