#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import re

NETLOC_LEN = len('github.com')

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Format a GitHub URL

def Fmt(netloc, path, query):

	# GitHub URL can be in one of the following formats:
	# ...
	if len(netloc) == NETLOC_LEN:
		# a) github.com/.+/<cve_id>/?.*
		if not re.search(r'/cve\-(199[0-9]|20[0-9]{2})-[0-9]{4}', path):
			# b) github.com/<vendor>/?
			# c) github.com/<vendor>/<product>(\.git)?/?.*
			match  = re.match(r'^/([^/]+)(/|$)', path)
			vendor = match[1] if match else ''
			if vendor:
				netloc = vendor + '.github.io'
				search = re.search(r'^[^/]+', path[len(match[0]):])
				product = search.group() if search else ''
				path = '/' + re.sub(r'\.git$', '', product) if product else ''
	else:
		# d) <vendor>.github.(com|io)/<product>/?.*
		# e) <product>.github.io/?.*
		netloc = re.sub(r'\.com$', '.io', netloc)
		i = path.find('/', 1)
		if i > 0:
			path = path[:i]

	query  = ''

	return netloc, path, query
