#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import re

NETLOC_LEN = len('gnome.org')

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Format a GNOME URL

def Fmt(netloc, path, query):

	prefix = netloc[:-(NETLOC_LEN + 1)] if len(netloc) > NETLOC_LEN else ''
	cut_prefix = True

	# Gnome URL can be in one of the following formats:
	# a) gnome.org/projects/<product>/?
	if not prefix:
		path = re.sub('^/projects/', '/', path)
	# b) wiki.gnome.org/(action/show/)?(design/)?(projects/|apps/)?<product>/?
	elif prefix == 'wiki':
		path = re.sub(
			'^/(projects|apps)/', '/', re.sub(
			'^/design/',          '/', re.sub(
			'^/action/show/',     '/', path))
			)
	# c) ftp.gnome.org/pub/gnome/(...)/<product>/?(<ver>)?/?
	elif prefix == 'ftp':
		search = re.search(r'/[0-9\.]+/?$', path)
		if search:
			path = path[:-len(search.group())]
		cut_prefix = False
	# d) git.gnome.org/(browse/|cgit/)?<product>/?
	elif prefix == 'git':
		path = re.sub(r'/(browse|cgit)/', '/', path)
	# e) gitlab.gnome.org/(...)
	elif prefix == 'gitlab':
		cut_prefix = False
	# f) projects.gnome.org/<product>/?
	# g) live.gnome.org/<product>/?
	else:
		pass

	# Final formatting
	if cut_prefix:
		netloc = 'gnome.org'
	if path and path[-1] == '/':
		path = path[:-1]

	return netloc, path, query
