#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import re

GNU_NETLOC_LEN = len('gnu.org')
SAVANNAH_NETLOC_LEN = len('savannah.')

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Format a GNU URL

def Fmt(netloc, path, query):

	netloc = netloc[:-GNU_NETLOC_LEN]
	nongnu = re.search(r'non$', netloc)
	if nongnu:
		netloc = netloc[:-len(nongnu.group())]
	if path:
		path = path[1:]
	fmt_path = True
	clear_query = True

	# GNU URL can be in one of the following formats:
	# a) git.savannah.gnu.org/cgit/<product>\.git/?.*
	# b) git.savannah.gnu.org/gitweb/?p=<product>.git [DEPRECATED]
	# c) download.savannah.gnu.org/releases/<product>
	# d) savannah.gnu.org/download/<product> (conv. to savannah.gnu.org/<product>)
	# e) savannah.gnu.org/(forum/.+|news)$
	# f) savannah.(non)?gnu.org/projects/<product>/?.*
	if re.search(r'savannah\.$', netloc):
		netloc = netloc[:-SAVANNAH_NETLOC_LEN]
		if netloc == 'git.':
			if path == 'gitweb':
				match = re.match(r'^p=(.+)\.git', query)
				if match:
					path = match[1]
			else:
				path = re.sub(
					r'\.git$', '', re.sub(
					r'/.*$',   '', re.sub(
					r'^cgit/', '', path))
					)
		elif netloc == 'download.':
			path = re.sub(r'^releases/', '', path)
		elif re.search(r'^(forum/.+|news)$', path):
			fmt_path = False
			clear_query = False
		else:
			path = re.sub(r'^projects/', '', path)
		netloc = ''
	# g) ftp://ftp.gnu.org/(non-)gnu/<product>/?.*
	elif netloc == 'ftp.':
		path = re.sub(r'/.+$', '', re.sub(r'^(non\-)?gnu/', '', path))
		netloc = ''
	# h) ftp://ftpmirror.gnu.org/<product>
	elif netloc == 'ftpmirror.':
		netloc = ''
	# i) (non?)gnu.org/(software|s)/<product>/?.*
	elif len(netloc) < 2:
		search = re.search(r'^(software|s)/', path)
		if search:
			path = re.sub(r'/.*', '', path[len(search.group()):])
		else:
			path = re.sub(r'/?(home\.[^/]+|news)$', '', path)
		netloc = ''
	# j) lists.gnu.org/archive/html/.*
	elif netloc == 'lists.':
		fmt_path = False
	# k) <product>.(non)?gnu.org/?(download|releases|wiki)?.* (conv. to <product>.(non)?gnu.org/?(releases|wiki)?.*)
	# l) <product>.gnu.org/bugzilla/.+
	elif re.search(r'^[^\.]+\.$', netloc):
		path = re.sub('^(releases|wiki)/?', '', path)
		if re.search(r'^bugzilla/.+$', path):
			fmt_path = False
			clear_query = False
		# For the 'gcc' product there are even more of useless path options
		elif netloc == 'gcc.':
			path = ''
		path = netloc[:-1] + (f'/{path}' if path else '')
		netloc = ''

	# Final formatting
	netloc = f'{netloc if netloc else ""}{"non" if nongnu else ""}gnu.org'
	if path:
		path = '/' + path
		if fmt_path:
			match = re.match(r'(/[^/]+)(/[^/]+)/', path)
			if match:
				path = match[1]
				if match[2] not in ('/refs', '/tags'):
					path += match[2]
	if clear_query:
		query  = ''

	return  netloc, path, query
