#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import re

NETLOC = 'mozilla.org'
NETLOC_LEN = len(NETLOC)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Check that given string is a language identifier (ru, en-us etc.)

def IsLangId(s):

	return bool(re.search(r'^[a-z]{2}(\-[a-z]{2})?$', s))

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Format a Mozilla URL

def Fmt(netloc, path, query):

	steps = [s for s in path.split('/') if s]

	if not steps:
		return netloc, path, query

	n_steps = len(steps)
	prefix  = netloc[:-(NETLOC_LEN + 1)] if len(netloc) > NETLOC_LEN else ''
	product = None

	# Processing Mozilla URLs of the following special formats:
	if prefix:
		# _ addons.mozilla.org/<lang_id>/(firefox|thunderbird)/addon/<product>/...
		# _ addons.mozilla.org/<lang_id>/<product>/...
		if prefix == 'addons' and IsLangId(steps[0]):
			if n_steps >= 4:
				if steps[2] == 'addon':
					product = steps[3]
			elif n_steps >= 2:
				product = steps[1]
			else:
				product = ''
		# _ wiki.mozilla.org/<product>
		# _ wiki.mozilla.org/release.*
		elif prefix == 'wiki':
			if not re.search(r'^release.*', steps[0]):
				product = steps[0]
		# _ ftp.mozilla.org/pub/mozilla.org/<product>/releases/...
		# _ ftp.mozilla.org/pub/<product>/releases/...
		elif prefix == 'ftp':
			if n_steps >= 2:
				if steps[1] == 'mozilla.org':
					if n_steps == 3 or steps[3] == 'releases':
						product = steps[2]
				else:
					if n_steps == 2 or steps[2] == 'releases':
						product = steps[1]
		# _ developer.mozilla.org/<lang_id>/docs/<product>/...
		# _ developer.mozilla.org/<lang_id>/docs/mozilla/projects/<product>/...
		elif prefix == 'developer':
			if n_steps >= 3 and IsLangId(steps[0]) and steps[1] == 'docs':
				if steps[2] == 'mozilla':
					if n_steps >= 5 and steps[3] == 'projects':
						product = steps[4]
				else:
					product = steps[2]
		# _ archive.mozilla.org/pub/<product>/...
		elif prefix == 'archive':
			if n_steps >= 2:
				product = steps[1]
		# _ <product>.mozilla.org/home
		elif steps[0] == 'home':
			path = ''
	else:
		# _ mozilla.org/<lang_id>/<product>/...
		# _ mozilla.org/<lang_id>/security/...
		if IsLangId(steps[0]):
			if n_steps >= 2:
				if steps[1] != 'security':
					product = steps[1]
			else:
				product = ''
		# _ mozilla.org/projects/security/pki/<product>
		# _ mozilla.org/projects/<product>
		elif steps[0] == 'projects':
			if n_steps >= 4 and steps[1] == 'security' and steps[2] == 'pki':
				product = steps[3]
			elif n_steps >= 2:
				product = steps[1]
			else:
				product = ''
		# _ mozilla.org/directory/<product>
		elif steps[0] == 'directory':
			product = steps[1]
		# _ mozilla.org/<product>
		else:
			product = steps[0]

	# Final formatting
	if product != None:
		netloc = f'{product}.{NETLOC}' if product else netloc
		path  = ''
		query = ''

	return netloc, path, ''
