#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import re

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Format an URL in which only first two parts of the path are matter

def FmtTwoPart(netloc, path, query):

	# Converting
	#	'<netloc>/<vendor>/<product>/?.*'
	# to
	#	'<netloc>/<vendor>/<product>'

	m = re.match(r'^/([^/]+)/([^/]+)/', path)
	if m:
		path = m.group(0)[:-1]

	return netloc, path, ''

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Format a Python URL

def FmtPython(netloc, path, query):

	# Converting
	#	'pypi.python.org/pypi/<product>'
	# to
	#	'pypi.org/project/<product>'

	m = re.match(r'^/pypi/([^/]+)', path)
	if m:
		path = '/project/' + m.group(1)

	return 'pypi.org', path, ''

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Format a Google URL

def FmtGoogle(netloc, path, query):

	# Converting
	#	'code.google.com/(archive/)?p/<product>/?.*'
	# to
	#	'code.google.com/p/<product>'

	path = re.sub(r'^/archive/', '/', path)

	return FmtTwoPart(netloc, path, query)
