#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import re

DOMAIN_NAME       = 'sourceforge.net'
DOMAIN_NAME_SHORT = 'sf.net'
DOMAIN_LEN_SHORT  = len(DOMAIN_NAME_SHORT)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Format a SourceForge URL

def Fmt(netloc, path, query):

	netloc  = re.sub(f'{DOMAIN_NAME}$', DOMAIN_NAME_SHORT, netloc)
	vendor  = ''
	product = ''

	# Sourceforge URL can be in one of the following formats:
	# a) sourceforge.net/(projects?|p)/<product>/?.*
	# b) <vendor>.sourceforge.net/index\.php?project=<product>
	# c) (<product>|<vendor>|downloads).sourceforge.net/
	#       (home/|wiki/| ... )?<product>/ ...
	# d) sourceforge.net/project/showfiles.php?group_id=[0-9]+
	if len(netloc) > (DOMAIN_LEN_SHORT + 1):
		subdomain = netloc[:-(DOMAIN_LEN_SHORT + 1)]
		if subdomain != 'downloads':
			vendor = subdomain
	match = re.match(r'project=(.+)', query) if query else None
	if match:
		product = match[1]
	else:
		match = re.match(r'^/(p/|projects?/)(?P<product>[^/]+)/?', path)
		if match:
			if match['product'] != 'showfiles':
				product = match['product']
		else:
			while True:
				filtered_path = re.sub(r'/(' \
					r'wiki|'          \
					r'home(page)?|'   \
					r'main_?(page)?|' \
					r'en|'            \
					r'changelog(\.txt)?|'   \
					r'(old|new)?releases?|' \
					r'[^/]+\.(tar|gz|zip)'  \
					r')(/|$)', '/', path
					)
				if filtered_path == path:
					break
				path = filtered_path
			if path:
				left = None if path[0] != '/' else 1
				right = None if path[-1] != '/' else -1
				product = path[left:right]
				path = ''

	if product:
		path = ''

	# If there is no vendor name but there is a product name then swap them
	if not vendor:
		vendor, product = product, ''

	# Final formatting
	netloc = f'{f"{vendor}." if vendor else ""}{DOMAIN_NAME_SHORT}'
	if product:
		path = f'/{product}'
		query = ''
	elif vendor:
		query = ''

	return netloc, path, query
